@@ -3,6 +3,7 @@ package Prima::Drawable;
 
 use strict;
 use warnings;
+use Prima;
 
 sub rect3d
 {
@@ -46,12 +47,14 @@ sub rect_focus
 	( $y, $y1) = ( $y1, $y) if $y > $y1;
 
 	$width = 1 if !defined $width || $width < 1;
-	my ( $cl, $cl2) = ( $canvas-> color, $canvas-> backColor);
+	my ( $cl, $cl2, $aa, $alpha) = ( $canvas-> color, $canvas-> backColor, $canvas-> antialias, $canvas-> alpha);
 	my $fp = $canvas-> fillPattern;
 	$canvas-> set(
 		fillPattern => fp::SimpleDots,
 		color       => cl::White,
 		backColor   => cl::Black,
+		antialias   => 0,
+		alpha       => 255,
 	);
 
 	if ( $width * 2 >= $x1 - $x or $width * 2 >= $y1 - $y) {
@@ -78,6 +81,9 @@ sub draw_text
 	$flags     = dt::Default unless defined $flags;
 	$tabIndent = 1 if !defined( $tabIndent) || $tabIndent < 0;
 
+	$x2 //= $x + 1;
+	$y2 //= $y + 1;
+
 	$x2 = int( $x2);
 	$x  = int( $x);
 	$y2 = int( $y2);
@@ -96,9 +102,9 @@ sub draw_text
 		(( $flags & dt::ExpandTabs    ) ? ( tw::ExpandTabs | tw::CalcTabs) : 0)
 	;
 
-	my @lines = @{$canvas-> text_wrap( $string,
-		( $flags & dt::NoWordWrap) ? 1000000 : $w,
-		$twFlags, $tabIndent
+	my @lines = @{$canvas-> text_wrap_shape( $string,
+		( $flags & dt::NoWordWrap) ? undef : $w,
+		options => $twFlags, tabs => $tabIndent
 	)};
 
 	my $tildes;
@@ -106,10 +112,6 @@ sub draw_text
 
 	return 0 unless scalar @lines;
 
-	if (($flags & dt::BidiText) && $Prima::Bidi::available) {
-		$_ = Prima::Bidi::visual($_) for @lines;
-	}
-
 	my @clipSave;
 	my $fh = $canvas-> font-> height +
 		(( $flags & dt::UseExternalLeading) ?
@@ -191,10 +193,10 @@ sub prelight_color
 		$coeff = 1/$coeff;
 	}
 	$coeff = ($coeff - 1) * 256;
-	my @channels = map { $_ & 0xff } cl::to_rgb($color);
+	my @channels = cl::to_rgb($color);
 	for (@channels) {
 		my $amp = ( 256 - $_ ) / 8;
-		$amp -= $amp if $coeff < 0;
+		$amp = -$amp if $coeff < 0;
 		$_ += $coeff + $amp;
 		$_ = 255 if $_ > 255;
 		$_ = 0   if $_ < 0;
@@ -206,7 +208,7 @@ sub text_split_lines
 {
 	my ($self, $text) = @_;
 	return ref($text) ?
-		@{ $self-> text_wrap( $text, 2_000_000_000, tw::NewLineBreak ) } :
+		@{ $self-> text_wrap( $text, -1, tw::NewLineBreak ) } :
 		split "\n", $text;
 }
 
@@ -222,7 +224,20 @@ sub new_gradient
 	return Prima::Drawable::Gradient->new(@_);
 }
 
-sub stroke_primitive
+sub new_aa_surface
+{
+	require Prima::Drawable::Antialias;
+	return Prima::Drawable::Antialias->new(@_);
+}
+
+sub new_glyph_obj
+{
+	shift;
+	require Prima::Drawable::Glyphs;
+	return Prima::Drawable::Glyphs->new(@_);
+}
+
+sub stroke_img_primitive
 {
 	my ( $self, $request ) = (shift, shift);
 	return 1 if $self->rop == rop::NoOper;
@@ -233,7 +248,7 @@ sub stroke_primitive
 	$path->translate(@offset);
 	$path->$request(@_);
 	my $ok = 1;
-	if ( $self->lineWidth == 0 ) {
+	if ( int($self->lineWidth + .5) == 0 ) {
 		# paths produce floating point coordinates and line end arcs,
 		# here we need internal pixel-wise plotting
 		for my $pp ( map { @$_ } @{ $path->points } ) {
@@ -280,7 +295,7 @@ sub stroke_primitive
 	return $ok;
 }
 
-sub fill_primitive
+sub fill_img_primitive
 {
 	my ( $self, $request ) = (shift, shift);
 	my $path = $self->new_path;
@@ -300,6 +315,157 @@ sub fill_primitive
 	return $ok;
 }
 
+sub stroke_imgaa_primitive
+{
+	my ( $self, $request ) = (shift, shift);
+	return 1 if $self->rop == rop::NoOper;
+	my $lp = $self->linePattern;
+	return 1 if $lp eq lp::Null && $self->rop2 == rop::NoOper;
+
+	my $aa = $self->new_aa_surface;
+	return 0 unless $aa->can_aa;
+
+	my $path = $self->new_path;
+	$path->$request(@_);
+	$path = $path->widen(
+		linePattern => ( $lp eq lp::Null) ? lp::Solid : $lp
+	);
+	my %save;
+	$save{fillPattern} = $self->fillPattern;
+	$save{fillMode}    = $self->fillMode;
+	$self->fillPattern(fp::Solid);
+	$self->fillMode(fm::Winding);
+	if ( $lp eq lp::Null ) {
+		$save{color} = $self->color;
+		$self->color($self->backColor);
+	}
+	my $ok = 1;
+	for ($path->points(fill => 1)) {
+		$ok &= $aa->fillpoly($_);
+		last unless $ok;
+	}
+	$self->$_($save{$_}) for keys %save;
+	return $ok;
+}
+
+sub fill_imgaa_primitive
+{
+	my ( $self, $request ) = (shift, shift);
+	my $path = $self->new_path;
+	$path->$request(@_);
+	my $aa = $self->new_aa_surface;
+	return 0 unless $aa->can_aa;
+	for ($path->points(fill => 1)) {
+		return 0 unless $aa->fillpoly($_);
+	}
+	return 1;
+}
+
+sub stroke_aa_primitive
+{
+	my ( $self, $request ) = (shift, shift);
+	return 1 if $self->rop == rop::NoOper;
+	my $lp = $self->linePattern;
+	return 1 if $lp eq lp::Null && $self->rop2 == rop::NoOper;
+
+	my $path = $self->new_path;
+	$path->$request(@_);
+	$path = $path->widen(
+		linePattern => ( $lp eq lp::Null) ? lp::Solid : $lp
+	);
+	my %save;
+	$save{fillPattern} = $self->fillPattern;
+	$save{fillMode}    = $self->fillMode;
+	$self->fillPattern(fp::Solid);
+	$self->fillMode(fm::Winding);
+	if ( $lp eq lp::Null ) {
+		$save{color} = $self->color;
+		$self->color($self->backColor);
+	}
+	my $ok = $path->fill;
+	$self->$_($save{$_}) for keys %save;
+	return $ok;
+}
+
+sub fill_aa_primitive
+{
+	my ( $self, $request ) = (shift, shift);
+	my $path = $self->new_path;
+	$path->$request(@_);
+	for ($path->points(fill => 1)) {
+		return 0 unless $self->fillpoly($_);
+	}
+	return 1;
+}
+
+sub text_shape_out
+{
+	my ( $self, $text, $x, $y, $rtl) = @_;
+	my %flags = (skip_if_simple => 1);
+	$flags{rtl} = $rtl if defined $rtl;
+	if ( my $glyphs = $self->text_shape($text, %flags)) {
+		$text = $glyphs;
+	}
+	return $self->text_out( $text, $x, $y);
+}
+
+sub get_text_shape_width
+{
+	my ( $self, $text, $flags) = @_;
+	my %flags = (skip_if_simple => 1);
+	$flags{rtl} = $flags & to::RTL if defined $flags;
+	if ( my $glyphs = $self->text_shape($text, %flags)) {
+		$text = $glyphs;
+	}
+	return $self->get_text_width( $text, $flags // 0);
+}
+
+sub text_wrap_shape
+{
+	my ( $self, $text, $width, %opt) = @_;
+
+	my $opt    = delete($opt{options}) // tw::Default;
+	my $shaped = $self-> text_shape( $text, %opt );
+	return $self->text_wrap( $text, $width // -1, $opt, delete($opt{tabs}) // 8) unless $shaped;
+	my $ret    = $self-> text_wrap( $text, $width // -1, $opt, delete($opt{tabs}) // 8, 0, -1, $shaped);
+
+	if (( my $justify = delete $opt{justify} ) && $ret && @$ret ) {
+		if (
+			$justify->{kashida} &&
+			!($opt & tw::ReturnChunks) &&
+			$text =~ /[\x{600}-\x{6ff}]/
+		) {
+			my $last = @$ret - ($opt & (tw::CalcMnemonic | tw::CollapseTilde)) ? -2 : -1;
+			for ( my $i = 0; $i < $last; $i++) {
+				if ( $opt & tw::ReturnGlyphs ) {
+					$$ret[$i]->justify_arabic($self, $text, $width, %opt, %$justify);
+				} elsif ( my $tx = $self->text_shape( $$ret[$i], %opt)) {
+					my $text = $tx->justify_arabic($self, $$ret[$i], $width, %opt, %$justify, as_text => 1);
+					$$ret[$i] = $text if defined $text;
+				}
+			}
+		}
+
+		if (
+			($justify->{letter} || $justify->{word}) &&
+			!($opt & tw::ReturnChunks)
+		) {
+			# do not justify last (or the only) line
+			my $last = @$ret - ($opt & (tw::CalcMnemonic | tw::CollapseTilde)) ? -3 : -2;
+			for ( my $i = 0; $i < $last; $i++) {
+				if ( $opt & tw::ReturnGlyphs ) {
+					$$ret[$i]->justify_interspace($self, $text, $width, %opt, %$justify);
+				} elsif ( my $tx = $self->text_shape( $$ret[$i], %opt)) {
+					my $text = $tx->justify_interspace($self, $$ret[$i], $width, %opt, %$justify, as_text => 1);
+					$$ret[$i] = $text if defined $text;
+				}
+			}
+		}
+	}
+
+	return $ret;
+}
+
 1;
 
 =head1 NAME
