@@ -2,6 +2,7 @@ package Prima::Drawable::Path;
 
 use strict;
 use warnings;
+use Prima;
 
 our $PI = 3.14159265358979323846264338327950288419716939937510;
 our $PI_2 = $PI / 2;
@@ -28,6 +29,8 @@ sub new
 		canvas          => $canvas,
 		commands        => [],
 		precision       => undef,
+		subpixel        => $opt{antialias} // (($canvas && ref($canvas)) ? $canvas->antialias : 0),
+		antialias       => 0,
 		%opt
 	}, $class;
 }
@@ -42,6 +45,8 @@ sub dup
 	);
 }
 
+sub canvas { $#_ ? $_[0]->{canvas} = $_[1] : $_[0]->{canvas} }
+
 sub cmd
 {
 	my ($self, $cmd, @param) = @_;
@@ -71,6 +76,7 @@ sub moveto           { shift->cmd('moveto', shift, shift, 0) }
 sub rmoveto          { shift->cmd('moveto', shift, shift, 1) }
 sub restore          { shift->cmd('restore') } # no checks for underflow here, to allow append paths
 sub precision        { shift->cmd(set => precision => shift) }
+sub antialias        { $#_ ? $_[0]->{antialias} = $_[1] : $_[0]->{antialias} }
 
 sub matrix_multiply
 {
@@ -112,7 +118,7 @@ sub shear
 {
 	my ( $self, $x, $y ) = @_;
 	$y //= $x;
-	$self-> matrix(1,$x,$y,1,0,0);
+	$self-> matrix(1,$y,$x,1,0,0);
 }
 
 
@@ -129,7 +135,7 @@ sub rotate
 sub line
 {
 	my $self = shift;
-	my $p = $#_ ? \@_ : $_[0];
+	my $p = $#_ ? [@_] : $_[0];
 	@$p % 2 and Carp::croak('bad parameters to line');
 	$self->cmd( line => $p );
 }
@@ -137,7 +143,7 @@ sub line
 sub rline
 {
 	my $self = shift;
-	my $p = $#_ ? \@_ : $_[0];
+	my $p = $#_ ? [@_] : $_[0];
 	@$p % 2 and Carp::croak('bad parameters to rline');
 	$self->rcmd( line => $p);
 }
@@ -156,6 +162,60 @@ sub rspline
 	$self-> rcmd( spline => $p, \%opt );
 }
 
+sub glyph
+{
+	my ($self, $ix, %opt) = @_;
+	return unless $self->{canvas};
+	my $outline = $self->{canvas}->render_glyph( $ix, %opt );
+	return unless $outline;
+	my $size = scalar(@$outline);
+	my @p;
+	my $fill = delete $opt{fill};
+	for ( my $i = 0; $i < $size; ) {
+		my $cmd = $outline->[$i++];
+		my $pts = $outline->[$i++] * 2;
+		my @pts = map { $outline->[$i++] / 64.0 } 0 .. $pts - 1;
+		if ( $cmd == ggo::Move ) {
+			$self->close unless $fill;
+			$self->moveto(@pts);
+		} elsif ( $cmd == ggo::Line ) {
+			$self->line([ @p, @pts ]);
+		} elsif ( $cmd == ggo::Conic ) {
+			$self->spline([ @p, @pts ]);
+		} elsif ( $cmd == ggo::Cubic ) {
+			$self->spline([ @p, @pts ], degree => 3 );
+		}
+		@p = @pts[-2,-1];
+	}
+	$self->close;
+}
+
+sub text
+{
+	my ($self, $text, %opt) = @_;
+	return unless my $c = $self->{canvas};
+	my $state = $c->get_paint_state;
+	unless ($state) {
+		return unless $c->begin_paint_info;
+	}
+
+	$self->translate( 0, $c->font->descent )
+		unless $opt{baseline} // $c->textOutBaseline;
+	my $cache   = $opt{cache} || {};
+	my $unicode = utf8::is_utf8($text);
+	for my $char ( split //, $text ) {
+		my $ix = ord($char);
+		$self->glyph($ix, %opt, unicode => $unicode);
+		my $r = $cache->{$char} //= do {
+			my $p = $c->get_font_abc($ix,$ix,$unicode);
+			$p->[0] + $p->[1] + $p->[2]
+		};
+		$self->translate($r,0);
+	}
+
+	$c->end_paint_info unless $state;
+}
+
 sub circular_arc
 {
 	my $self = shift;
@@ -217,10 +277,11 @@ sub chord
 sub lines
 {
 	my $self = shift;
-	my $p = $#_ ? \@_ : $_[0];
+	my $p = $#_ ? [@_] : $_[0];
 	@$p % 4 and Carp::croak('bad parameters to lines');
 	for ( my $i = 0; $i < @$p; $i += 4 ) {
-		$self->cmd( line => [ @$p[ $i .. $i + 3 ] ] );
+		$self->moveto(@$p[ $i .. $i + 1 ]);
+		$self->cmd( line   => [ @$p[ $i .. $i + 3 ] ] );
 	}
 }
 
@@ -245,27 +306,77 @@ sub sector
 		restore;
 }
 
+sub round_rect
+{
+	my ( $self, $x, $y, $x1, $y1, $maxd) = @_;
+	( $x1, $x) = ( $x, $x1) if $x > $x1;
+	( $y1, $y) = ( $y, $y1) if $y > $y1;
+	my ( $dx, $dy) = ( $x1 - $x, $y1 - $y);
+	$dx = $maxd if $dx > $maxd;
+	$dy = $maxd if $dy > $maxd;
+	my $d = ( $dx < $dy ) ? $dx : $dy;
+	my $r = $self->{subpixel} ? $d/2 : int($d/2);
+#  plots roundrect:
+# A'        B'
+#  /------\
+#  |A    B|
+#  |      |  arcs cannot have diameter larger than $maxd
+#  |C    D|
+#  \------/
+# C'        D'
+	my @r = (
+		# coordinates of C and B, so A=r[0,3],B=r[2,3],C=r[0,1],D=[2,1]
+		$x + $r, $y + $r,
+		$x1 - $r, $y1 - $r,
+		# coordinates of C' and B'
+		$x, $y,
+		$x1, $y1,
+	);
+	$self-> line( @r[2,7,0,7]) if $r[0] < $r[2];
+	$self-> arc( @r[0,3], $d, $d, 90, 180);
+	$self-> line( @r[4,3,4,1]) if $r[1] < $r[3];
+	$self-> arc( @r[0,1], $d, $d, 180, 270);
+	$self-> line( @r[0,5,2,5]) if $r[0] < $r[2];
+	$self-> arc( @r[2,1], $d, $d, 270, 360);
+	$self-> line( @r[6,1,6,3]) if $r[1] < $r[3];
+	$self-> arc( @r[2,3], $d, $d, 0, 90);
+	return $self;
+}
+
+sub new_array { shift->{subpixel} ? Prima::array->new_double : Prima::array->new_int }
+
 sub points
 {
-	my $self = shift;
+	my ($self, %opt) = @_;
 	unless ( $self->{points} ) {
 		local $self->{stack} = [];
 		local $self->{curr}  = {
 			matrix => [ identity ],
-			( map { $_, $self->{$_} } qw(precision ) )
+			( map { $_, $self->{$_} } qw(precision) )
 		};
-		$self->{points} = [ Prima::array->new_int ];
+		$self->{points} = [[ $self->new_array ]];
 		my $c = $self->{commands};
 		for ( my $i = 0; $i < @$c; ) {
 			my ($cmd,$len) = @$c[$i,$i+1];
 			$self-> can("_$cmd")-> ( $self, @$c[$i+2..$i+$len+1] );
 			$i += $len + 2;
-			
 		}
-		@{$self->{points}} = grep { @$_ > 2 } @{$self->{points}};
+		for my $ppp ( @{$self->{points}}) {
+			@$ppp = grep { @$_ > 2 } @$ppp;
+		}
 		$self->{last_matrix} = $self->{curr}->{matrix};
 	}
 
+	if ( $opt{fill} ) {
+		my @ret;
+		for my $ppp ( @{ $self->points } ) {
+			my $arr = $self->new_array;
+			Prima::array::append( $arr, $_ ) for @$ppp;
+			push @ret, $arr if @$arr > 2;
+		}
+		return @ret;
+	}
+
 	return $self->{points};
 }
 
@@ -279,7 +390,9 @@ sub last_matrix
 sub last_point
 {
 	for ( reverse @{ shift->{points} }) {
-		return $$_[-2], $$_[-1] if @$_;
+		for ( reverse @$_ ) {
+			return $$_[-2], $$_[-1] if @$_;
+		}
 	}
 	return 0,0;
 }
@@ -288,16 +401,8 @@ sub matrix_apply
 {
 	my $self   = shift;
 	my ($ref, $points) = $#_ ? (0, [@_]) : (1, $_[0]);
-	my $m  = $self->{curr}->{matrix};
-	my @ret;
-	for ( my $i = 0; $i < @$points; $i += 2 ) {
-		my ( $x, $y ) = @{$points}[$i,$i+1];
-		push @ret,
-			$$m[A] * $x + $$m[C] * $y + $$m[X],
-			$$m[B] * $x + $$m[D] * $y + $$m[Y]
-			;
-	}
-	return $ref ? \@ret : @ret;
+	my $ret = Prima::Drawable->render_polyline( $points, matrix => $self->{curr}->{matrix} );
+	return $ref ? $ret : @$ret;
 }
 
 sub _save
@@ -340,39 +445,46 @@ sub _relative
 	$m->[Y] += $ly - $y0;
 }
 
-sub _moveto
+sub  _moveto
 {
 	my ( $self, $mx, $my, $rel) = @_;
 	($mx, $my) = $self->matrix_apply($mx, $my);
 	my ($lx, $ly) = $rel ? $self->last_point : (0,0);
-	push @{$self->{points}}, Prima::array->new_int;
-	push @{$self->{points}->[-1]}, int($lx + $mx + .5), int($ly + $my + .5);
+	my $arr = $self->new_array;
+	push @$arr, $self->{subpixel} ? ($lx + $mx, $ly + $my) : (int($lx + $mx + .5), int($ly + $my + .5));
+	push @{$self->{points}->[-1]}, $arr;
 }
 
-sub _open { push @{shift->{points}}, Prima::array->new_int }
+sub _open { push @{$_[0]->{points}}, [$_[0]->new_array] }
 
 sub _close
 {
 	my $self = shift;
 	my $p = $self->{points};
-	push @{$p->[-1]}, $p->[0][0], $p->[0][1]
-		if @{$p->[-1]} && ($p->[0][0] != $p->[-1][-2] || $p->[0][1] != $p->[-1][-1]);
-	push @$p, Prima::array->new_int;
+	return unless @$p;
+	my $l = $p->[-1]->[-1];
+	push @$l, $$l[0], $$l[1] if @$l && ($$l[0] != $$l[-2] || $$l[1] != $$l[-1]);
+	push @$p, [$self->new_array];
 }
 
 sub _line
 {
 	my ( $self, $line ) = @_;
-	push @{ $self->{points}->[-1] }, map { int($_ + .5) } @{ $self-> matrix_apply( $line ) };
+	if ( $self->{subpixel} ) {
+		push @{ $self->{points}->[-1]->[-1] }, @{ $self-> matrix_apply( $line ) };
+	} else {
+		push @{ $self->{points}->[-1]->[-1] }, map { int($_ + .5) } @{ $self-> matrix_apply( $line ) };
+	}
 }
 
 sub _spline
 {
 	my ( $self, $points, $options ) = @_;
-	Prima::array::append( $self->{points}->[-1],
+	Prima::array::append( $self->{points}->[-1]->[-1],
 		Prima::Drawable->render_spline(
 			$self-> matrix_apply( $points ),
-			%$options
+			%$options,
+			integer => !$self->{subpixel},
 		)
 	)
 }
@@ -449,10 +561,11 @@ sub _arc
 
 	my %xopt;
 	$xopt{precision} = $self->{curr}->{precision} if defined $self->{curr}->{precision};
+	$xopt{integer}   = !$self->{subpixel};
 
 	for my $set ( @$nurbset ) {
 		my ( $points, @options ) = @$set;
-		Prima::array::append( $self->{points}->[-1],
+		Prima::array::append( $self->{points}->[-1]->[-1],
 			Prima::Drawable->render_spline(
 				$self-> matrix_apply( $points ),
 				@options,
@@ -464,18 +577,47 @@ sub _arc
 
 sub stroke {
 	return 0 unless $_[0]->{canvas};
-	for ( @{ $_[0]->points }) {
-		return 0 unless $_[0]->{canvas}->polyline($_);
+	for ( map { @$_ } @{ $_[0]->points }) {
+		if ( $_[0]->{antialias} && !$_[0]->{canvas}->antialias) {
+			return 0 unless $_[0]->{canvas}->new_aa_surface->polyline($_);
+		} else {
+			return 0 unless $_[0]->{canvas}->polyline($_);
+		}
 	}
 	return 1;
 }
 
 sub fill {
-	return 0 unless $_[0]->{canvas};
-	for ( @{ $_[0]->points }) {
-		return 0 unless $_[0]->{canvas}->fillpoly($_);
+	my ( $self, $fillMode ) = @_;
+	return 0 unless my $c = $self->{canvas};
+	my @p = $self->points(fill => 1);
+	my $ok = 1;
+	my $save;
+	if ( defined $fillMode ) {
+		$save = $c->fillMode;
+		$c->fillMode($fillMode);
 	}
-	return 1;
+	for ( @p ) {
+		if ( $self->{antialias} && !$_[0]->{canvas}->antialias) {
+			last unless $ok &= $c->new_aa_surface->fillpoly($_);
+		} else {
+			last unless $ok &= $c->fillpoly($_);
+		}
+	}
+	$c->fillMode($save) if defined $save;
+	return $ok;
+}
+
+sub fill_stroke
+{
+	my ( $self, $fillMode ) = @_;
+	return 0 unless my $c = $self->{canvas};
+	my $color = $c->color;
+	$c->color( $c-> backColor);
+	my $ok = $self->fill($fillMode);
+	$c->color( $color );
+	$ok &= $self->stroke;
+	return $ok;
 }
 
 sub flatten
@@ -514,6 +656,7 @@ sub flatten
 
 			my %xopt;
 			$xopt{precision} = $self->{curr}->{precision} if defined $self->{curr}->{precision};
+			$xopt{integer}   = !$self->{subpixel};
 			my $polyline;
 			my $nurbset = $self->arc2nurbs( $from, $to);
 			for my $set ( @$nurbset ) {
@@ -546,9 +689,60 @@ sub flatten
 	);
 }
 
+sub contours
+{
+	my $self = shift;
+	my @ret;
+	for my $pp ( map { @$_ } @{ $self->points } ) {
+		my @contour;
+		next if @$pp < 2;
+		my $closed = $pp->[0] == $pp->[-2] && $pp->[1] == $pp->[-1];
+		for ( my $i = 0; $i < @$pp - 2; $i += 2 ) {
+			my @a = @{$pp}[$i,$i+1];
+			my @b = @{$pp}[$i+2,$i+3];
+		
+			my ( $delta_y, $delta_x, $dir);
+			next if $a[0] == $b[0] && $a[1] == $b[1] && @$pp > 4;
+
+			$delta_y = $b[1] - $a[1];
+			$delta_x = $b[0] - $a[0];
+			$dir = 1 if abs($delta_y) > abs($delta_x);
+	
+			my ( $curr_maj, $curr_min, $to_maj, $delta_maj, $delta_min ) = $dir ? 
+				($a[1], $a[0], $b[1], $delta_y, $delta_x) :
+				($a[0], $a[1], $b[0], $delta_x, $delta_y);
+			my $inc_maj = ($delta_maj != 0) ?
+				(abs($delta_maj)==$delta_maj ? 1 : -1) : 0;
+			my $inc_min = ($delta_min != 0) ?
+				(abs($delta_min)==$delta_min ? 1 : -1) : 0;
+			$delta_maj = abs($delta_maj);
+			$delta_min = abs($delta_min);
+			my $d      = ($delta_min * 2) - $delta_maj;
+			my $d_inc1 = ($delta_min * 2);
+			my $d_inc2 = (($delta_min - $delta_maj) * 2);
+	
+			while(1) {
+				my @p = $dir ? ($curr_min, $curr_maj) : ($curr_maj, $curr_min);
+				push @contour, @p;
+				last if $curr_maj == $to_maj;
+				$curr_maj += $inc_maj;
+				if ($d < 0) {
+					$d += $d_inc1;
+				} else {
+					$d += $d_inc2;
+					$curr_min += $inc_min;
+				}
+			}
+			pop @contour, pop @contour if $closed || $i > 0;
+		}
+		push @ret, \@contour if @contour;
+	}
+	return @ret;
+}
+
 sub poly2patterns
 {
-	my ($pp, $lp, $lw) = @_;
+	my ($pp, $lp, $lw, $int) = @_;
 	$lw = 1 if $lw < 1;
 	my @steps = map { 1 + $lw * (ord($_) - 1 ) } split '', $lp;
 #	print "$lw: steps: @steps\n";
@@ -580,7 +774,7 @@ sub poly2patterns
 				$dx = $b[0] - $a[0];
 				$dy = $b[1] - $a[1];
 				my $dl = $dx * $dx + $dy * $dy;
-				$pixlen = (($dl < 1024 ) ?
+				$pixlen = (($dl < 1024 && $int ) ?
 					$sqrt[$dl + .5] //= sqrt(int($dl + .5)) :
 					sqrt($dl)
 				);
@@ -588,7 +782,7 @@ sub poly2patterns
 				@r = ($pixlen > 0) ? 
 					($dx / $pixlen, $dy / $pixlen):
 					(1,1);
-				$pixlen = int( $pixlen + .5 );
+				$pixlen = int( $pixlen + .5 ) if $int;
 				if (($i == $#$p - 1 && !$closed) || ($pixlen == 0)) {
 					$pixlen++;
 				} else {
@@ -603,6 +797,7 @@ sub poly2patterns
 			}
 			($draw, $black) = ( $advance > 0 ) ? ($advance, 0) : ($strokelen, $strokecolor);
 #			print "draw:$advance/$strokelen pixlen:$pixlen plotted:$plotted black:$black\n";
+			my $next_seg_advance = $black ? $lw - 1 : 1;
 			if ( $draw < $pixlen ) {
 				$plotted += $draw;
 				@b1 = ($draw == 1) ? @a1 : (
@@ -612,19 +807,19 @@ sub poly2patterns
 #				print "pix($black): @a1 -> @b1\n";
 				push @$segment, @a1, @b1 if $black;
 				$pixlen -= $draw;
-				$advance += ($advance > 0) ? -$draw : ($lw-1);
+				$advance += ($advance > 0) ? -$draw : $next_seg_advance;
 				@a1 = ( $b1[0] + $r[0], $b1[1] + $r[1]);
 #				print "new adv to @a1? =$advance\n";
 				($new_point, $new_stroke) = (0,1);
 			} elsif ( $draw == $pixlen ) {
 				push @$segment, @a1, @b if $black;
 				$new_stroke = $new_point = 1;
-				$advance += ($advance > 0) ? -$draw : ($lw-1);
+				$advance += ($advance > 0) ? -$draw : $next_seg_advance;
 #				print "=: pix($black): @a1 -> @b\n";
 				$joiner = $black;
 			} elsif ( $black && $draw == 1 && $pixlen <= 0 )  {
 				$new_point = $new_stroke = 1;
-				$advance = $lw-1;
+				$advance = $next_seg_advance;
 #				print "skip tail\n";
 			} else {
 #				print ">: pix($black): @a1 -> @b\n";
@@ -672,13 +867,15 @@ sub widen
 		$opt;
 	} qw(lineWidth lineJoin lineEnd linePattern);
 
-	my $pp = $self->points;
+	my $pp = [ map { @$_ } @{$self->points} ];
 	return $dst if $lp eq lp::Null;
-	$pp = poly2patterns($pp, $lp, $lw) if $lp ne lp::Solid;
+	$pp = poly2patterns($pp, $lp, $lw, !$self->{subpixel}) if $lp ne lp::Solid;
 
-	if ( $lw <= 1 ) {
+	if ( $lw < 1 && !$self->{subpixel} ) {
 		for my $p ( @$pp ) {
 			$dst->line($p);
+			$dst->line([map { @{$p}[-2*$_,-2*$_+1] } 1..@$p/2 ])
+				if $lp eq lp::Solid;
 			$dst->open;
 		}
 		return $dst;
@@ -759,13 +956,14 @@ sub widen
 					($prev, $next) = ($i - 2, 0);
 				}
 				my ($xo,$yo,$xa,$ya,$xb,$yb) = @$p[$i,$i+1,$prev,$prev+1,$next,$next+1];
-				my $theta = atan2( $yo - $ya, $xo - $xa );
-        	        	my $alpha = atan2( $yb - $yo, $xb - $xo ) - $theta;
+				my $dya = $yo - $ya;
+				my $dxa = $xo - $xa;
+				my $dyb = $yb - $yo;
+				my $dxb = $xb - $xo;
+				my $theta = atan2( $dya, $dxa );
+        	        	my $alpha = atan2( $dyb, $dxb ) - $theta;
 				$alpha += $PI * (($alpha > 0) ? -1 : 1);
 				# next if $alpha == 0.0; # XXX
-				my $_lj = $lj;
-				$_lj = lj::Bevel if
-					$_lj == lj::Miter && ($alpha == 0 || $ml < abs( 1 / sin($alpha/2)));
 				my $sign = ( $alpha > 0) ? -1 : 1;
 				my ( $in, $out) = ($alpha > 0) ? (\@u,\@d) : (\@d,\@u);
 				my ( $dx1, $dy1, $dx2, $dy2) = map { $sign * $lw2 * $_ } (
@@ -774,10 +972,18 @@ sub widen
 					cos($theta + $alpha + $PI_2),
 					sin($theta + $alpha + $PI_2)
 				);
+				my $_lj = $lj;
+				my $dmin = 3;
+				$_lj = lj::Miter if $_lj != lj::Miter &&
+					abs($dya) < $dmin && abs($dxa) < $dmin && abs($dyb) < $dmin && abs($dxb) < $dmin;
+				$_lj = lj::Bevel if
+					$_lj == lj::Miter && ($alpha == 0 || $ml < abs( 1 / sin($alpha/2)));
 				if ($i == 0) {
 					@$firstin = ( $xo + $dx1, $yo + $dy1);
 					$firstsign = $sign;
 				}
+				next if $dxa == 0 && $dya == 0;
+				next if $dxb == 0 && $dyb == 0;
 				push @$in, [ line => [ $xo + $dx1, $yo + $dy1 ]];
 				push @$in, [ line => [ $xo - $dx2, $yo - $dy2 ]];
 				if ( $_lj == lj::Miter) {
@@ -838,10 +1044,10 @@ sub widen
 sub extents
 {
 	my $self = shift;
-	my $pp = $self->points;
-	return unless @$pp;
-	my ( $x1, $y1, $x2, $y2 ) = @{$pp->[0]}[0,1,0,1];
-	for my $p ( @$pp ) {
+	my @pp = map { @$_ } @{ $self->points };
+	return unless @pp;
+	my ( $x1, $y1, $x2, $y2 ) = @{$pp[0]}[0,1,0,1];
+	for my $p ( @pp ) {
 		for ( my $i = 2; $i < $#$p; $i+=2) {
 			my ($x, $y) = @{$p}[$i,$i+1];
 			$x1 = $x if $x < $x1;
@@ -865,17 +1071,18 @@ sub clip
 	$p->clear;
 	$p->set(%opt) if scalar keys %opt;
 	$p->translate($tx, $ty);
-	$p->fillpoly($_) for @{ $self->{points} };
+	$p->fillpoly($_) for $self->points(fill => 1);
 	return $p->image;
 }
 
 sub region
 {
-	my ($self, $mode) = @_;
+	my ($self, $mode, $rgnop) = @_;
 	my $reg;
 	$mode //= fm::Winding | fm::Overlay;
-	$reg ? $reg->combine($_, rgnop::Union) : ($reg = $_)
-		for map { Prima::Region->new( polygon => $_, fillMode => $mode) } @{ $self->points };
+	$rgnop //= rgnop::Union;
+	$reg ? $reg->combine($_, $rgnop) : ($reg = $_)
+		for map { Prima::Region->new( polygon => $_, fillMode => $mode) } $self->points(fill => 1);
 	return $reg;
 }
 
@@ -898,15 +1105,16 @@ to be used for drawing or clipping shapes.
 	# draws elliptic spiral
 	my ( $d1, $dx ) = ( 0.8, 0.05 );
 	$canvas-> new_path->
-		scale(200, 100)->
 		rotate(45)->
+		translate(200, 100)->
+		scale(200, 100)->
 		arc( 0, 0, $d1 + $dx * 0, $d1 + $dx * 1, 0, 90)->
 		arc( 0, 0, $d1 + $dx * 2, $d1 + $dx * 1, 90, 180)->
 		arc( 0, 0, $d1 + $dx * 2, $d1 + $dx * 3, 180, 270)->
 		arc( 0, 0, $d1 + $dx * 4, $d1 + $dx * 3, 270, 360)->
 	stroke;
 
-=for podview <img src="path.gif">
+=for podview <img src="Prima/path.gif">
 
 =for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/path.gif">
 
@@ -952,6 +1160,12 @@ Adds chord to the path. Is there only for compatibility with C<Prima::Drawable>.
 
 Adds full ellipse to the path.
 
+=item glyph INDEX, %OPTIONS
+
+Adds glyph outline to the path. C<%OPTIONS> are passed as is to L<Prima::Drawable/renger_glyph>.
+Note that filled glyphs require C<fillMode> without the C<fm::Overlay> bit set and C<fill> option set
+to generate proper shapes with holes.
+
 =item line, rline @POINTS
 
 Adds a polyline to path
@@ -974,6 +1188,10 @@ point of the previous primitive, or (0,0) if there's none.
 
 Adds rectangle to the path. Is there only for compatibility with C<Prima::Drawable>.
 
+=item round_rect X1, Y1, X2, Y2, MAX_DIAMETER
+
+Adds round rectangle to the path.
+
 =item sector CENTER_X, CENTER_Y, DIAMETER_X, DIAMETER_Y, ANGLE_START, ANGLE_END
 
 Adds sector to the path. Is there only for compatibility with C<Prima::Drawable>.
@@ -982,6 +1200,23 @@ Adds sector to the path. Is there only for compatibility with C<Prima::Drawable>
 
 Adds B-spline to path. See L<Prima::Drawable/spline> for C<%OPTIONS> descriptions.
 
+=item text TEXT, %OPTIONS
+
+Adds C<TEXT> to the path. C<%OPTIONS> are same as in L<Prima::Drawable/render_glyph>, 
+except that C<unicode> is deduced automatically based on whether C<TEXT> has utf8 bit
+on or off; and an extra option C<cache> with a hash can be used to speed up the function
+with subsequent calls. C<baseline> option is same as L<Prima::Drawable/textOutBaseline>.
+
+=back
+
+=head2 Properties
+
+=over
+
+=item canvas DRAWABLE
+
+Provides access to the attached drawable object
+
 =back
 
 =head2 Transformations
@@ -1015,6 +1250,12 @@ and when applied to 2D coordinates, is calculated as
 
 Selects current precision for splines and arcs. See L<Prima::Drawable/spline>, C<precision> entry.
 
+=item antialias BOOLEAN
+
+Turns on and off slow but more visually pleasant antialiased drawing mode.
+
+Default: false
+
 =item restore
 
 Pops the stack entry and replaces the current matrix and graphic properties with it.
@@ -1035,6 +1276,12 @@ Adds shearing to the current matrix
 
 Adds scaling to the current matrix
 
+=item subpixel BOOLEAN
+
+Turns on and off slow but more precise floating-point calculation mode
+
+Default: depends on canvas antialiasing mode
+
 =item translate X, Y = X
 
 Adds offset to the current matrix
@@ -1053,6 +1300,11 @@ create an array of points that can be used for actual drawing.
 Returns 1-bit image with clipping mask of the path. C<%options> can be used to
 pass C<fillMode> property that affects the result of the filled shape.
 
+=item contours
+
+Same as L<points> but further reduces lines into a 8-connected set of points,
+suitable to be traced pixel-by-pixel.
+
 =item extents
 
 Returns 2 points that box the path.
@@ -1061,9 +1313,15 @@ Returns 2 points that box the path.
 
 Return CTM resulted after running all commands
 
-=item fill
+=item fill fillMode=undef
 
-Paints a filled shape over the path
+Paints a filled shape over the path. If C<fillMode> is set, it is used instead of the one
+selected on the canvas.
+
+=item fill_stroke fillMode=undef
+
+Paints a filled shape over the path with back color. If C<fillMode> is set, it is used instead of the one
+selected on the canvas. Thereafter, draws a polyline over the path.
 
 =item flatten PRESCALE 
 
@@ -1073,15 +1331,17 @@ pixels, to be suitable for direct send to the polyline() API call. If PRESCALE
 factor is set, it is used instead to premultiply coordinates of arc anchor
 points used to render the lines.
 
-=item points
+=item points FOR_FILL_POLY=0
 
 Runs all accumulated commands, and returns rendered set of points, suitable
-for further calls to C<Prima::Drawable::polyline> and C<Prima::Drawable::fillpoly>.
+for further calls to either C<Prima::Drawable::polyline> or C<Prima::Drawable::fillpoly>
+depending on the C<FOR_FILL_POLY> flag.
 
-=item region MODE=fm::Winding|fm::Overlay
+=item region MODE=fm::Winding|fm::Overlay, RGNOP=rgnop::Union
 
 Creates a region object from polygonal shape. If MODE is set, applies fill mode
-(see L<Drawable/fillMode> for more).
+(see L<Prima::Drawable/fillMode> for more); if RGNOP is set, applies region set operation
+(see L<Prima::Region/combine>).
 
 =item stroke
 
