@@ -1,20 +1,18 @@
 package Prima::Label;
 use vars qw(@ISA);
 @ISA = qw(Prima::Widget);
-
-use Carp;
-use Prima::Const;
-use Prima::Classes;
-use Prima::Bidi qw(is_bidi);
 use strict;
 use warnings;
+use Carp;
+use Prima;
 
 sub profile_default
 {
 	my $font = $_[ 0]-> get_default_font;
+	my $rtl  = $::application-> textDirection;
 	return {
 		%{$_[ 0]-> SUPER::profile_default},
-		alignment      => $Prima::Bidi::default_direction_rtl ? ta::Right : ta::Left,
+		alignment      => $rtl ? ta::Right : ta::Left,
 		autoHeight     => 0,
 		autoWidth      => 1,
 		focusLink      => undef,
@@ -25,13 +23,14 @@ sub profile_default
 		showAccelChar  => 0,
 		showPartial    => 1,
 		tabStop        => 0,
+		textDirection  => $rtl,
+		textJustify    => 0,
 		valignment     => ta::Top,
 		widgetClass    => wc::Label,
 		wordWrap       => 0,
 	}
 }
 
-
 sub profile_check_in
 {
 	my ( $self, $p, $default) = @_;
@@ -49,6 +48,8 @@ sub profile_check_in
 		exists $p-> {rect} ||
 		( exists $p-> {top} && exists $p-> {bottom})
 	);
+	$p-> {alignment} = ( $p->{textDirection} // $default->{textDirection} ) ?
+		ta::Right : ta::Left unless exists $p->{alignment};
 	$self-> SUPER::profile_check_in( $p, $default);
 	my $vertical = exists $p-> {vertical} ?
 		$p-> {vertical} :
@@ -59,12 +60,17 @@ sub profile_check_in
 sub init
 {
 	my $self = shift;
+	$self->{lock} = 1;
+	$self->{textLines} = 0;
 	my %profile = $self-> SUPER::init(@_);
+	$self->textJustify($profile{textJustify});
 	$self-> {$_} = $profile{$_} for qw(
-		alignment valignment autoHeight autoWidth
+		textDirection alignment valignment autoHeight autoWidth
 		wordWrap focusLink showAccelChar showPartial hotKey
 	);
 	$self-> check_auto_size;
+	delete $self->{lock};
+	$self->reset_lines;
 	return %profile;
 }
 
@@ -90,7 +96,6 @@ sub on_paint
 
 	my $fh = $canvas-> font-> height;
 	my $ta = $self-> {alignment};
-	my $wx = $self-> {widths};
 	my $ws = $self-> {words};
 	my ($starty,$ycommon) = (0, scalar @{$ws} * $fh);
 
@@ -107,25 +112,42 @@ sub on_paint
 	my $i;
 	my $paintLine = !$self-> {showAccelChar} && defined($tl) && $tl < scalar @{$ws};
 
+	my (@wx, @wss);
+	my %justify = ( %{$self->textJustify}, rtl => $self->textDirection);
+	for ( my $i = 0; $i < @$ws; $i++) {
+		my $s = $canvas->text_shape($ws->[$i], %justify );
+		if ( $s && ref($s)) {
+			$s->justify(
+				$canvas, $ws->[$i], $size[0],
+				%justify,
+				($i == $#$ws) ? (letter => 0, word => 0) : ()
+			);
+			push @wss, $s;
+		} else {
+			push @wss, $ws->[$i];
+		}
+		push @wx, $canvas->get_text_width($wss[-1]);
+	}
+
 	unless ( $self-> enabled) {
 		$canvas-> color( $self-> light3DColor);
-		for ( $i = 0; $i < scalar @{$ws}; $i++) {
+		for ( $i = 0; $i < @wss; $i++) {
 			my $x = 0;
 			if ( $ta == ta::Center) {
-				$x = ( $size[0] - $$wx[$i]) / 2;
+				$x = ( $size[0] - $wx[$i]) / 2;
 			} elsif ( $ta == ta::Right) {
-				$x = $size[0] - $$wx[$i];
+				$x = $size[0] - $wx[$i];
 			}
-			$canvas-> text_out( $$ws[$i], $x + 1, $y - 1);
+			$canvas-> text_out( $wss[$i], $x + 1, $y - 1);
 			$y -= $fh;
 		}
 		$y   = $starty;
 		if ( $paintLine) {
 			my $x = 0;
 			if ( $ta == ta::Center) {
-				$x = ( $size[0] - $$wx[$tl]) / 2;
+				$x = ( $size[0] - $wx[$tl]) / 2;
 			} elsif ( $ta == ta::Right) {
-				$x = $size[0] - $$wx[$tl];
+				$x = $size[0] - $wx[$tl];
 			}
 			$canvas-> line(
 				$x + $self-> {tildeStart} + 1, $starty - $fh * $tl - 1,
@@ -135,20 +157,20 @@ sub on_paint
 	}
 
 	$canvas-> color( $clr[0]);
-	for ( $i = 0; $i < scalar @{$ws}; $i++) {
+	for ( $i = 0; $i < @wss; $i++) {
 		my $x = 0;
 		if ( $ta == ta::Center) {
-			$x = ( $size[0] - $$wx[$i]) / 2;
+			$x = ( $size[0] - $wx[$i]) / 2;
 		} elsif ( $ta == ta::Right) {
-			$x = $size[0] - $$wx[$i];
+			$x = $size[0] - $wx[$i];
 		}
-		$canvas-> text_out( $$ws[$i], $x, $y);
+		$canvas-> text_out( $wss[$i], $x, $y);
 		$y -= $fh;
 	}
 	if ( $paintLine) {
 		my $x = 0;
-		if ( $ta == ta::Center) { $x = ( $size[0] - $$wx[$tl]) / 2; }
-		elsif ( $ta == ta::Right) { $x = $size[0] - $$wx[$tl]; }
+		if ( $ta == ta::Center) { $x = ( $size[0] - $wx[$tl]) / 2; }
+		elsif ( $ta == ta::Right) { $x = $size[0] - $wx[$tl]; }
 		$canvas-> line(
 			$x + $self-> {tildeStart}, $starty - $fh * $tl,
 			$x + $self-> {tildeEnd},   $starty - $fh * $tl
@@ -157,11 +179,10 @@ sub on_paint
 }
 
 
-sub text
+sub set_text
 {
-	return $_[0]-> SUPER::text unless $#_;
 	my $self = $_[0];
-	$self-> SUPER::text( $_[1]);
+	$self-> SUPER::set_text( $_[1]);
 	$self-> check_auto_size;
 	$self-> repaint;
 }
@@ -249,24 +270,25 @@ sub set_valignment
 sub reset_lines
 {
 	my ($self, $nomaxlines) = @_;
+	return if $self->{lock};
 
 	my @res;
 	my $maxLines = int($self-> height / $self-> font-> height);
 	$maxLines++ if $self-> {showPartial} and (($self-> height % $self-> font-> height) > 0);
 
 	my $opt   = tw::NewLineBreak|tw::ReturnLines|tw::WordBreak|tw::CalcMnemonic|tw::ExpandTabs|tw::CalcTabs;
-	my $width = 1000000;
+	my $width;
 	$opt |= tw::CollapseTilde unless $self-> {showAccelChar};
 	$width = $self-> width if $self-> {wordWrap};
 
 	$self-> begin_paint_info;
 
-	my $lines = $self-> text_wrap( $self-> text, $width, $opt);
+	my $lines = $self-> text_wrap_shape( $self-> text, $width,
+		options => $opt,
+		rtl     => $self->textDirection,
+	);
 	my $lastRef = pop @{$lines};
 
-	if ( $Prima::Bidi::enabled) {
-		$_ = Prima::Bidi::visual($_) for grep { is_bidi $_ } @$lines;
-	}
 	$self-> {textLines} = scalar @$lines;
 	for( qw( tildeStart tildeEnd tildeLine)) {$self-> {$_} = $lastRef-> {$_}}
 
@@ -274,10 +296,6 @@ sub reset_lines
 	splice( @{$lines}, $maxLines) if scalar @{$lines} > $maxLines && !$nomaxlines;
 	$self-> {words} = $lines;
 
-	my @len;
-	for ( @{$lines}) { push @len, $self-> get_text_width( $_); }
-	$self-> {widths} = [@len];
-
 	$self-> end_paint_info;
 }
 
@@ -368,6 +386,34 @@ sub autoHeight    {($#_)?($_[0]-> set_auto_height(  $_[1]))   :return $_[0]-> {a
 sub wordWrap      {($#_)?($_[0]-> set_word_wrap(    $_[1]))   :return $_[0]-> {wordWrap}     }
 sub hotKey        { $#_ ? $_[0]->{hotKey} = $_[1] : $_[0]->{hotKey} }
 
+sub textDirection
+{
+	return $_[0]-> {textDirection} unless $#_;
+	my ( $self, $td ) = @_;
+	$self-> {textDirection} = $td;
+	$self-> text( $self-> text );
+	$self-> alignment( $td ? ta::Right : ta::Left );
+}
+
+sub textJustify
+{
+	return {%{$_[0]-> {textJustify}}} unless $#_;
+	my $self = shift;
+	if ( @_ % 2 ) {
+		my $tj = shift;
+		my %tj = ref($tj) ? %$tj : map { ($_, !!$tj) } qw(letter word kashida);
+		$tj{$_} //= 0 for qw(letter word kashida min_kashida);
+		if ( ref($tj) ) {
+			$tj{$_} = $$tj{$_} for qw(min_text_to_space_ratio);
+		}
+		$self->{textJustify} = \%tj;
+	} else {
+		my %h = @_;
+		$self->{textJustify}->{$_} = $h{$_} for keys %h;
+	}
+	$self-> text( $self-> text );
+}
+
 1;
 
 =pod
@@ -402,7 +448,7 @@ is useful for dialog design.
 	);
 	run Prima;
 
-=for podview <img src="label.gif" cut=1>
+=for podview <img src="label.gif">
 
 =for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/label.gif">
 
@@ -474,6 +520,25 @@ lines are drawn.
 
 Default value: 1
 
+=item textJustify $BOOL | { letter => 0, word => 0, kashida => 0, min_kashida => 0 } | %VALUES
+
+If set, justifies wrapped text according to the option passwd in the hash
+( see L<Prima::Drawable::Glyphs/arabic_justify> and L<Prima::Drawable::Glyphs/interspace_justify> ).
+Can accept three forms:
+
+If anonymous hash is used, overwrites all the currently defined options.
+
+If C<$BOOL> is used, treated as a shortcut for C<< { letter => $BOOL, word => $BOOL, kashida => $BOOL } >>;
+consequent get-calls return full hash, not the C<$BOOL>.
+
+If C<%VALUES> form is used, overwrites only values found in C<%VALUES>.
+
+Only actual when C<wordWrap> is set.
+
+=item textDirection BOOLEAN
+
+If set, indicates RTL text direction.
+
 =item wordWrap BOOLEAN
 
 If 1, the text is wrapped if it can not be horizontally fit in the
