@@ -12,9 +12,9 @@ use Prima::PS::Printer;
 =head1 DESCRIPTION
 
 Realizes the Prima printer interface to PostScript level 2 document language
-through Prima::PS::Drawable module. Allows different user profiles to be
-created and managed with GUI setup dialog. The module is designed to be
-compliant with Prima::Printer interface.
+through Prima::PS::PostScript module and PDF v1.4 through the Prima::PS::PDF
+module. Allows different user profiles to be created and managed with GUI setup
+dialog. The module is designed to be compliant with Prima::Printer interface.
 
 Also contains convenience classes (File, LPR, Pipe) for non-GUI use.
 
@@ -24,11 +24,13 @@ Also contains convenience classes (File, LPR, Pipe) for non-GUI use.
 
 	my $x;
 	if ( $preview) {
-		$x = Prima::PS::Pipe-> new( command => 'gv -');
+		$x = Prima::PS::Pipe-> new( command => 'gv $');
 	} elsif ( $print_in_file) {
 		$x = Prima::PS::File-> new( file => 'out.ps');
 	} elsif ( $print_on_device) {
 		$x = Prima::PS::LPR-> new( args => '-d colorprinter');
+	} elsif ( $print_pdf_file ) {
+		$x = Prima::PS::PDF::File-> new( file => 'out.pdf');
 	} else {
 		$x = Prima::PS::FileHandle-> new( handle => \*STDOUT );
 	}
@@ -44,27 +46,335 @@ use warnings;
 use Prima;
 use Prima::Utils;
 use IO::Handle;
-use Prima::PS::Drawable;
+use Prima::PS::PostScript;
+use Prima::PS::PDF;
+use Prima::PS::TempFile;
+use Prima::PS::Setup;
+use Prima::MsgBox;
+
+package Prima::PS::Printer::Common;
+use Prima::sys::FS;
+
+our $unix = Prima::Application-> get_system_info-> {apc} == apc::Unix;
+
+our @spooler_types = qw(lpr file cmd fh);
+
+our %pageSizes = ( # in points
+	'A0'  => [ 2391, 3381],
+	'A1'  => [ 1688, 2391],
+	'A2'  => [ 1193, 1688],
+	'A3'  => [ 843, 1193],
+	'A4'  => [ 596, 843],
+	'A5'  => [ 419, 596],
+	'A6'  => [ 297, 419],
+	'A7'  => [ 209, 297],
+	'A8'  => [ 146, 209],
+	'A9'  => [ 103, 146],
+	'B0'  => [ 2929, 4141],
+	'B1'  => [ 2069, 2929],
+	'B10' => [ 89, 126],
+	'B2'  => [ 1463, 2069],
+	'B3'  => [ 1034, 1463],
+	'B4'  => [ 729, 1034],
+	'B5'  => [ 516, 729],
+	'B6'  => [ 362, 516],
+	'B7'  => [ 257, 362],
+	'B8'  => [ 180, 257],
+	'B9'  => [ 126, 180],
+	'C5E' => [ 462, 650],
+	'US Common #10 Envelope' => [ 297, 684],
+	'DLE'       => [ 311, 624],
+	'Executive' => [ 541, 721],
+	'Folio'     => [ 596, 937],
+	'Ledger'    => [ 1227, 792],
+	'Legal'     => [ 613, 1011],
+	'Letter'    => [ 613, 792],
+	'Tabloid'   => [ 792, 1227],
+);
+
+sub data
+{
+	return $_[0]-> {data} unless $#_;
+	my ( $self, $dd) = @_;
+	my $p = $self-> {data};
+	$self-> resolution( $p-> {resolution}, $p-> {resolution})
+		if exists $dd-> {resolution};
+	$self-> scale( $p-> {scaling}, $p-> {scaling}) if exists $dd-> {scaling};
+	$self-> reversed( $p-> {portrait} ? 0 : 1) if exists $dd-> {portrait};
+	$self-> pageSize(
+		@{exists($pageSizes{$p-> {page}}) ?
+			$pageSizes{$p-> {page}} : $pageSizes{A4}}
+	) if exists $dd-> {page};
+	$self-> grayscale( $p-> {color} ? 0 : 1) if exists $dd-> {color};
+}
+
+sub defaultData
+{
+	return $_[0]-> {defaultData} unless $#_;
+	my ( $self, $dd) = @_;
+	for ( keys %$dd) {
+		$self-> {defaultData}-> {$_} = $dd-> {$_};
+	}
+}
+
+sub _begin_doc
+{
+	my ( $self, $docName) = @_;
+	return 0 if $self-> get_paint_state;
+
+	$self-> {spoolingFailed} = undef;
+	$self-> {spoolSTDERR} = undef;
+	if ($self-> {data}-> {spoolerType} ne 'fh') {
+		$self-> {spoolHandle} = undef;
+	} else {
+		return 0 unless $self->{spoolHandle};
+	}
+
+	my $backend = $self->{backend};
+
+	if ( $self-> {data}-> {spoolerType} eq 'file') {
+		if ( $self-> {gui}) {
+			require Prima::Dialog::FileDialog;
+			my $f = Prima::save_file(
+				defaultExt => $self->{fileExtension},
+				filter     => [
+					[ $self->{formatDescription} => "*.$self->{fileExtension}"], 
+					[ 'All files' => '*.*']
+				],
+				text       => 'Print to file',
+			);
+			return 0 unless defined $f;
+			my $h = IO::Handle-> new;
+			unless ( open $h, ">", $f) {
+				undef $h;
+				message("Error opening $f:$!");
+				goto AGAIN;
+			}
+			binmode $h;
+			$self-> {spoolHandle} = $h;
+			$self-> {spoolName}   = $f;
+		} else { # no gui
+			my $h = IO::Handle-> new;
+			my $f = $self-> {data}-> {spoolerData};
+			unless ( open $h, ">", $f) {
+				undef $h;
+				return 0;
+			}
+			binmode $h;
+			$self-> {spoolHandle} = $h;
+			$self-> {spoolName}   = $f;
+		}
+		unless ( $backend->can('begin_doc')->( $self, $docName)) {
+			unlink( $self-> {spoolName});
+			close( $self-> {spoolHandle});
+			return 0;
+		}
+		return 1;
+	} elsif ( $self-> {data}-> {spoolerType} eq 'cmd' && $self->{data}->{spoolerData} =~ /\$/) {
+		my $f = Prima::PS::TempFile->new(unlink => 0, warn => !$self->{gui});
+		unless ( defined $f ) {
+			message("Error creating temporary file: $!") if $self-> {gui};
+			return 0;
+		}
+		$self-> {spoolTmpFile} = $f;
+		$self-> {spoolHandle}  = $f->{fh};
+		$self-> {spoolName}    = $f->{filename};
+	}
+	return $backend->can('begin_doc')->( $self, $docName);
+}
+
+my ( $sigpipe);
+
+sub __end
+{
+	my ($self, $aborted) = @_;
+	close( $self-> {spoolHandle}) if $self-> {spoolHandle} && $self-> {data}-> {spoolerType} ne 'fh';
+	if ( $self-> {data}-> {spoolerType} ne 'file') {
+		defined($sigpipe) ? $SIG{PIPE} = $sigpipe : delete($SIG{PIPE});
+	}
+	$self-> {spoolHandle} = undef if $self->{data}->{spoolerType} ne 'fh';
+	$self-> {spoolingFailed} = undef;
+
+	if ( $self->{data}->{spoolerType} eq 'cmd' && $self->{data}->{spoolerData} =~ /\$/) {
+		unless ( $aborted ) {
+			my $cmd = $self->{data}->{spoolerData};
+			my $tmp = $self->{spoolName};
+			$cmd =~ s/\$/$tmp/g;
+			if ( $self->{gui} && $cmd !~ />2/) {
+				$self->{spoolSTDERR} = Prima::PS::TempFile->new_filename;
+				$cmd .= " 2>$self->{spoolSTDERR}";
+			}
+			$self-> show_msg("Error running '$cmd'") if system $cmd;
+		}
+		$self->{spoolTmpFile}->remove;
+		undef $self->{spoolTmpFile};
+	}
+
+	unlink $self->{spoolSTDERR} if defined $self->{spoolSTDERR};
+	$self-> {spoolSTDERR} = undef;
+
+	$sigpipe = undef;
+}
+
+sub _end_doc
+{
+	my $self = $_[0];
+	my $backend = $self->{backend};
+	$backend->can('end_doc')->($self);
+	$self-> __end(0);
+}
+
+sub _abort_doc
+{
+	my $self = $_[0];
+	my $backend = $self->{backend};
+	$backend->can('abort_doc')->($self);
+	$self-> __end(1);
+	unlink $self-> {spoolName} if $self-> {data}-> {spoolerType} eq 'file';
+}
+
+sub show_msg
+{
+	my ( $self, $msg ) = @_;
+	return unless $self->{gui};
+
+	if (
+		defined($self->{spoolSTDERR}) &&
+		(open my $f, "<", $self->{spoolSTDERR} )
+	) {
+		local $/;
+		$msg .= ": " . <$f>;
+		close $f;
+	}
+
+	message($msg);
+}
+
+sub _spool
+{
+	my ( $self, $data) = @_;
+
+	my $piped = 0;
+
+	return 0 if $self->{spoolingFailed};
+
+	if ( $self-> {data}-> {spoolerType} ne 'file' && !$self-> {spoolHandle}) {
+		my @cmds;
+		if ( $self-> {data}-> {spoolerType} eq 'lpr') {
+			push( @cmds, map { $_ . ' ' . $self-> {data}-> {spoolerData}} qw(
+				lp lpr /bin/lp /bin/lpr /usr/bin/lp /usr/bin/lpr));
+		} else {
+			push( @cmds, $self-> {data}-> {spoolerData});
+		}
+		my $ok = 0;
+		$sigpipe = $SIG{PIPE};
+		$SIG{PIPE} = 'IGNORE';
+		$self->{spoolSTDERR} = $self->{gui} ? Prima::PS::TempFile->new_filename : undef;
+		CMDS: for ( @cmds) {
+			$piped = 0;
+			my $f = IO::Handle-> new;
+			my $cmd = "|$_";
+			$cmd .= " 2>$self->{spoolSTDERR}" if $self->{gui};
+			next unless open $f, $cmd;
+			binmode $f;
+			$f-> autoflush(1);
+			$piped = 1 unless print $f $data;
+			close( $f), next if $piped;
+			$ok = 1;
+			$self-> {spoolHandle} = $f;
+			$self-> {spoolName}   = $_;
+			last;
+		}
+		$self-> show_msg("Error printing to '$cmds[0]'") if !$ok;
+		$self-> {spoolingFailed} = 1 if !$ok;
+		return $ok;
+	}
+
+	if (
+		!(print {$self-> {spoolHandle}} $data) ||
+		( $piped && $self-> {data}-> {spoolerType} ne 'file' )
+	) {
+		$self-> show_msg("Error printing to '$self->{spoolName}'");
+		$self-> {spoolingFailed} = 1;
+		return 0;
+	}
+
+	return 1;
+}
+
+
+sub install_api
+{
+	my $pkg = shift;
+	no strict 'refs';
+	no warnings 'redefine';
+	for my $sym ( qw(begin_doc end_doc abort_doc spool)) {
+		*{$pkg . '::' . $sym} = UNIVERSAL::can(__PACKAGE__, '_' . $sym);
+	}
+}
+
+sub options
+{
+	my $self = shift;
+
+	if ( 0 == @_) {
+		return qw(
+			Color Resolution PaperSize Scaling Orientation 
+		), keys %{$self->{data}->{devParms}};
+	} elsif ( 1 == @_) {
+		# get value
+		my $v = shift;
+		my $d = $self-> {data};
+
+		if ( $v eq 'Orientation') {
+			return $d->{portrait} ? 'Portrait' : 'Landscape'
+		} elsif ( $v eq 'Color') {
+			return $d->{color} ? 'Color' : 'Monochrome'
+		} else {
+			$v = 'page' if $v eq 'PaperSize';
+			$v = lcfirst $v;
+			return $d-> {$v};
+		}
+	} else {
+		my %newdata;
+		my $successfully_set = 0;
+		while ( @_ ) {
+			# set value
+			my ( $opt, $val) = ( shift, shift);
+			my $d = $self-> {data};
+			if ( $opt eq 'Orientation') {
+				next unless $val =~ /^(?:(Landscape)|(Portrait))$/;
+				$newdata{portrait} = $2 ? 1 : 0;
+			} elsif ( $opt eq 'Color') {
+				next unless $val =~ /^(?:(Color)|(Monochrome))$/;
+				$newdata{color} = $1 ? 1 : 0;
+			} else {
+				$opt = lcfirst $opt;
+				$opt = 'page' if $opt eq 'PaperSize';
+				next unless exists $d->{$opt};
+				$newdata{$opt} = $val;
+			}
+			$successfully_set++;
+		}
+		$self-> data( \%newdata);
+		return $successfully_set;
+	}
+}
 
 package Prima::PS::Printer;
-use vars qw(@ISA %pageSizes $unix);
-@ISA = qw(Prima::PS::Drawable);
-
-$unix = Prima::Application-> get_system_info-> {apc} == apc::Unix;
-
-use constant lpr  => 0;
-use constant file => 1;
-use constant cmd  => 2;
-use constant fh   => 3;
+use base qw(Prima::PS::PostScript Prima::PS::Printer::Common);
 
+Prima::PS::Printer::Common::install_api(__PACKAGE__);
 
 sub profile_default
 {
 	my $def = $_[ 0]-> SUPER::profile_default;
 	my %prf = (
-		resFile       => Prima::Utils::path . '/Printer',
-		printer       => undef,
-		gui           => 1,
+		resFile           => Prima::Utils::path . '/Printer',
+		printer           => undef,
+		gui               => 1,
+		fileExtension     => 'ps',
+		formatDescription => 'PostScript',
 		defaultData   => {
 			color          => 1,
 			resolution     => 300,
@@ -73,9 +383,7 @@ sub profile_default
 			isEPS          => 0,
 			scaling        => 1,
 			portrait       => 1,
-			useDeviceFonts => 1,
-			useDeviceFontsOnly => 0,
-			spoolerType    => $unix ? lpr : file,
+			spoolerType    => $unix ? 'lpr' : 'file',
 			spoolerData    => '',
 			devParms       => {
 				MediaType                   => '',
@@ -96,16 +404,22 @@ sub profile_default
 	return $def;
 }
 
+sub profile_check_in
+{
+	my ( $self, $p, $default) = @_;
+	$p->{userData}->{$_} = $p->{$_} for qw(resolution reversed rotate scaling);
+}
 
 sub init
 {
 	my $self = shift;
 	my %profile = $self-> SUPER::init(@_);
+	$self-> {backend} = 'Prima::PS::PostScript';
 	$self-> {defaultData} = {};
 	$self-> {data} = {};
 	$self-> $_($profile{$_}) for qw(defaultData resFile);
 	$self-> {printers} = {};
-	$self-> {gui} = $profile{gui};
+	$self-> {$_} = $profile{$_} for qw(gui fileExtension formatDescription );
 
 	my $pr = $profile{printer} if defined $profile{printer};
 
@@ -115,6 +429,11 @@ sub init
 		close F;
 		eval "$fc";
 		$self-> {printers} = $fc if !$@ && defined($fc) && ref($fc) eq 'HASH';
+		for my $v ( values %{ $self->{printers} } ) {
+			if ( $v->{spoolerType} =~ /^\d+$/ ) { # old format
+				$v->{spoolerType} = $Prima::PS::Printer::Common::spooler_types[ $v->{spoolerType} ];
+			}
+		}
 	}
 
 	unless ( scalar keys %{$self-> {printers}}) {
@@ -122,13 +441,16 @@ sub init
 		if ( $unix) {
 			$self-> import_printers( 'printers', '/etc/printcap');
 			$self-> {printers}-> {GhostView} = deepcopy( $self-> {defaultData});
-			$self-> {printers}-> {GhostView}-> {spoolerType} = cmd;
-			$self-> {printers}-> {GhostView}-> {spoolerData} = 'gv -';
+			$self-> {printers}-> {GhostView}-> {spoolerType} = 'cmd';
+			$self-> {printers}-> {GhostView}-> {spoolerData} = 'gv $';
 		}
-		$self-> {printers}-> {File} = deepcopy( $self-> {defaultData});
-		$self-> {printers}-> {File}-> {spoolerType} = file;
+		$self-> {printers}-> {'Save as PostScript'} = deepcopy( $self-> {defaultData});
+		$self-> {printers}-> {'Save as PostScript'}-> {spoolerType} = 'file';
 	}
 
+	if ( exists $self->{printers}->{File} && ! exists $self->{printers}->{'Save as PostScript'}) {
+		$self-> {printers}-> {'Save as PostScript'} = delete $self->{printers}->{File};
+	}
 
 	unless ( defined $pr) {
 		if ( defined  $self-> {printers}-> {'Default printer'}) {
@@ -140,6 +462,13 @@ sub init
 	}
 
 	$self-> printer( $pr);
+	$self-> $_( $profile{userData}->{$_} ) for
+		grep { defined $profile{userData}->{$_} }
+		qw(reversed rotate);
+	$self-> resolution( @{$profile{userData}->{resolution}} )
+		if defined $profile{userData}->{resolution};
+	$self-> scale( ($profile{userData}->{scaling}) x 2)
+		if defined $profile{userData}->{scaling};
 	return %profile;
 }
 
@@ -173,7 +502,7 @@ sub import_printers
 			$j++;
 		}
 		$self-> {$slot}-> {$n} = deepcopy( $self-> {defaultData});
-		$self-> {$slot}-> {$n}-> {spoolerType} = lpr;
+		$self-> {$slot}-> {$n}-> {spoolerType} = 'lpr';
 		$self-> {$slot}-> {$n}-> {spoolerData} = "-d $_";
 		push @ret, $n;
 	}
@@ -183,27 +512,21 @@ sub import_printers
 sub printers
 {
 	my $self = $_[0];
+
 	my @res;
-	for ( keys %{$self-> {printers}}) {
+	for ( sort keys %{$self-> {printers}}) {
 		my $d = $self-> {printers}-> {$_};
 		push @res, {
 			name    => $_,
 			device  =>
-			(( $d-> {spoolerType} == lpr ) ? "lp $d->{spoolerData}"  :
-			(( $d-> {spoolerType} == file) ? 'file' : $d-> {spoolerData})),
+			(( $d-> {spoolerType} eq 'lpr' ) ? "lp $d->{spoolerData}"  :
+			(( $d-> {spoolerType} eq 'file') ? 'file' : $d-> {spoolerData})),
 			defaultPrinter =>  ( $self-> {current} eq $_) ? 1 : 0,
 		},
 	}
 	return \@res;
 }
 
-sub resolution
-{
-	return $_[0]-> SUPER::resolution unless $#_;
-	$_[0]-> raise_ro("resolution") if @_ != 3; # pass inherited call
-	shift-> SUPER::resolution( @_);
-}
-
 sub resFile
 {
 	return $_[0]-> {resFile} unless $#_;
@@ -215,6 +538,13 @@ sub resFile
 	$self-> {resFile} = $fn;
 }
 
+sub resolution
+{
+	return $_[0]-> SUPER::resolution unless $#_;
+	$_[0]-> raise_ro("resolution") if @_ != 3; # pass inherited call
+	shift-> SUPER::resolution( @_);
+}
+
 sub printer
 {
 	return $_[0]-> {current} unless $#_;
@@ -245,19 +575,10 @@ sub data
 			$p-> {devParms}-> {$_} = $dv-> {$_};
 		}
 	}
-	$self-> SUPER::resolution( $p-> {resolution}, $p-> {resolution})
-		if exists $dd-> {resolution};
-	$self-> scale( $p-> {scaling}, $p-> {scaling}) if exists $dd-> {scaling};
+
+	$self-> SUPER::data( $dd );
 	$self-> isEPS( $p-> {isEPS}) if exists $dd-> {isEPS};
-	$self-> reversed( $p-> {portrait} ? 0 : 1) if exists $dd-> {portrait};
-	$self-> pageSize(
-		@{exists($pageSizes{$p-> {page}}) ?
-			$pageSizes{$p-> {page}} : $pageSizes{A4}}
-	) if exists $dd-> {page};
-	if ( exists $dd-> {page}) {
-		$self-> useDeviceFonts( $p-> {useDeviceFonts});
-		$self-> useDeviceFontsOnly( $p-> {useDeviceFontsOnly});
-	}
+
 	if ( defined $dv) {
 		my %dp = %{$p-> {devParms}};
 		for ( keys %dp) {
@@ -280,7 +601,6 @@ sub data
 		}
 		$self-> pageDevice( \%dp);
 	}
-	$self-> grayscale( $p-> {color} ? 0 : 1) if exists $dd-> {color};
 }
 
 sub defaultData
@@ -289,9 +609,7 @@ sub defaultData
 	my ( $self, $dd) = @_;
 	my $dv = $dd-> {devParms};
 	delete $dd-> {devParms};
-	for ( keys %$dd) {
-		$self-> {defaultData}-> {$_} = $dd-> {$_};
-	}
+	$self-> SUPER::defaultData($dd);
 	if ( defined $dv) {
 		for ( keys %$dv) {
 			$self-> {defaultData}-> {devParms}-> {$_} = $dv-> {$_};
@@ -299,254 +617,71 @@ sub defaultData
 	}
 }
 
-
-%pageSizes = ( # in points
-	'A0'  => [ 2391, 3381],
-	'A1'  => [ 1688, 2391],
-	'A2'  => [ 1193, 1688],
-	'A3'  => [ 843, 1193],
-	'A4'  => [ 596, 843],
-	'A5'  => [ 419, 596],
-	'A6'  => [ 297, 419],
-	'A7'  => [ 209, 297],
-	'A8'  => [ 146, 209],
-	'A9'  => [ 103, 146],
-	'B0'  => [ 2929, 4141],
-	'B1'  => [ 2069, 2929],
-	'B10' => [ 89, 126],
-	'B2'  => [ 1463, 2069],
-	'B3'  => [ 1034, 1463],
-	'B4'  => [ 729, 1034],
-	'B5'  => [ 516, 729],
-	'B6'  => [ 362, 516],
-	'B7'  => [ 257, 362],
-	'B8'  => [ 180, 257],
-	'B9'  => [ 126, 180],
-	'C5E' => [ 462, 650],
-	'US Common #10 Envelope' => [ 297, 684],
-	'DLE'       => [ 311, 624],
-	'Executive' => [ 541, 721],
-	'Folio'     => [ 596, 937],
-	'Ledger'    => [ 1227, 792],
-	'Legal'     => [ 613, 1011],
-	'Letter'    => [ 613, 792],
-	'Tabloid'   => [ 792, 1227],
-);
-
 sub deepcopy
 {
 	my %h = %{$_[0]};
-	$h{devParms} = {%{$h{devParms}}};
+	$h{devParms} = {%{$h{devParms}}} if exists $h{devParms};
 	return \%h;
 }
 
 sub setup_dialog
 {
 	return unless $_[0]-> {gui};
-	eval "use Prima::PS::Setup"; die "$@\n" if $@;
 	$_[0]-> sdlg_exec;
 }
 
-sub begin_doc
-{
-	my ( $self, $docName) = @_;
-	return 0 if $self-> get_paint_state;
-
-	if ($self-> {data}-> {spoolerType} != fh) {
-		$self-> {spoolHandle} = undef;
-	} else {
-		return 0 unless $self->{spoolHandle};
-	}
-
-	if ( $self-> {data}-> {spoolerType} == file) {
-		if ( $self-> {gui}) {
-			eval "use Prima::MsgBox"; die "$@\n" if $@;
-			my $f = Prima::MsgBox::input_box( 'Print to file', 'Output file name:', '', mb::OKCancel, buttons => {
-				mb::OK, {
-				modalResult => undef,
-				onClick => sub {
-					$_[0]-> clear_event;
-					my $f = $_[0]-> owner-> InputLine1-> text;
-					if ( -f $f) {
-						return 0 if Prima::MsgBox::message(
-							"File $f already exists. Overwrite?",
-							mb::Warning|mb::OKCancel) != mb::OK;
-					} else {
-						unless ( open F, "> $f") {
-							Prima::MsgBox::message(
-							"Error opening $f:$!", mb::Error|mb::OK);
-							return 0;
-						}
-						close F;
-						unlink $f;
-					}
-					$_[0]-> owner-> modalResult( mb::OK);
-					$_[0]-> owner-> end_modal;
-			}}});
-			return 0 unless defined $f;
-			my $h = IO::Handle-> new;
-			unless ( open $h, "> $f") {
-				undef $h;
-				Prima::message("Error opening $f:$!");
-				goto AGAIN;
-			}
-			$self-> {spoolHandle} = $h;
-			$self-> {spoolName}   = $f;
-		} else { # no gui
-			my $h = IO::Handle-> new;
-			my $f = $self-> {data}-> {spoolerData};
-			unless ( open $h, "> $f") {
-				undef $h;
-				return 0;
-			}
-			$self-> {spoolHandle} = $h;
-			$self-> {spoolName}   = $f;
-		}
-		unless ( $self-> SUPER::begin_doc( $docName)) {
-			unlink( $self-> {spoolName});
-			close( $self-> {spoolHandle});
-			return 0;
-		}
-		return 1;
-	}
-
-	return $self-> SUPER::begin_doc( $docName);
-}
-
-my ( $sigpipe);
-
-sub __end
-{
-	my $self = $_[0];
-	close( $self-> {spoolHandle}) if $self-> {spoolHandle} && $self-> {data}-> {spoolerType} != fh;
-	if ( $self-> {data}-> {spoolerType} != file) {
-		defined($sigpipe) ? $SIG{PIPE} = $sigpipe : delete($SIG{PIPE});
-	}
-	$self-> {spoolHandle} = undef if $self->{data}->{spoolerType} != fh;
-	$sigpipe = undef;
-}
-
-sub end_doc
-{
-	my $self = $_[0];
-	$self-> SUPER::end_doc;
-	$self-> __end;
-}
-
-sub abort_doc
-{
-	my $self = $_[0];
-	$self-> SUPER::abort_doc;
-	$self-> __end;
-	unlink $self-> {spoolName} if $self-> {data}-> {spoolerType} == file;
-}
-
-sub spool
-{
-	my ( $self, $data) = @_;
-
-	my $piped = 0;
-	if ( $self-> {data}-> {spoolerType} != file && !$self-> {spoolHandle}) {
-		my @cmds;
-		if ( $self-> {data}-> {spoolerType} == lpr) {
-			push( @cmds, map { $_ . ' ' . $self-> {data}-> {spoolerData}} qw(
-				lp lpr /bin/lp /bin/lpr /usr/bin/lp /usr/bin/lpr));
-		} else {
-			push( @cmds, $self-> {data}-> {spoolerData});
-		}
-		my $ok = 0;
-		$sigpipe = $SIG{PIPE};
-		$SIG{PIPE} = 'IGNORE';
-		CMDS: for ( @cmds) {
-			$piped = 0;
-			my $f = IO::Handle-> new;
-			next unless open $f, "|$_";
-			$f-> autoflush(1);
-			$piped = 1 unless print $f $data;
-			close( $f), next if $piped;
-			$ok = 1;
-			$self-> {spoolHandle} = $f;
-			$self-> {spoolName}   = $_;
-			last;
-		}
-		Prima::message("Error printing to '$cmds[0]'") if !$ok && $self-> {gui};
-		return $ok;
-	}
-
-	if ( !(print {$self-> {spoolHandle}} $data) ||
-			( $piped && $self-> {data}-> {spoolerType} != file )
-		) {
-		Prima::message( "Error printing to '$self->{spoolName}'") if $self-> {gui};
-		return 0;
-	}
-
-	return 1;
-}
-
 sub options
 {
 	my $self = shift;
 
 	if ( 0 == @_) {
-		return qw(
-			Color Resolution PaperSize Copies Scaling Orientation
-			UseDeviceFonts UseDeviceFontsOnly EPS
-		), keys %{$self->{data}->{devParms}};
+		return
+			$self-> SUPER::options,
+			qw(EPS),
+			keys %{$self->{data}->{devParms}};
 	} elsif ( 1 == @_) {
 		# get value
 		my $v = shift;
 		my $d = $self-> {data};
 		return $d->{devParms}->{$v} if exists $d->{devParms}->{$v};
 
-		if ( $v eq 'Orientation') {
-			return $d->{portrait} ? 'Portrait' : 'Landscape'
-		} elsif ( $v eq 'Color') {
-			return $d->{color} ? 'Color' : 'Monochrome'
-		} elsif ( $v eq 'EPS') {
+		if ( $v eq 'EPS') {
 			return $d->{isEPS} ? '1' : '0'
 		} else {
-			$v = 'page' if $v eq 'PaperSize';
-			$v = lcfirst $v;
-			return $d-> {$v};
+			return $self-> SUPER::options($v);
 		}
+
 	} else {
-		my %newdata;
 		my $successfully_set = 0;
+		my (%newdata, %superdata);
 		while ( @_ ) {
 			# set value
 			my ( $opt, $val) = ( shift, shift);
 			my $d = $self-> {data};
 			if ( exists $d->{devParms}->{$opt}) {
 				$newdata{devParms}->{$opt} = $val;
-			} elsif ( $opt eq 'Orientation') {
-				next unless $val =~ /^(?:(Landscape)|(Portrait))$/;
-				$newdata{portrait} = $2 ? 1 : 0;
-			} elsif ( $opt eq 'Color') {
-				next unless $val =~ /^(?:(Color)|(Monochrome))$/;
-				$newdata{color} = $1 ? 1 : 0;
 			} else {
-				$opt = lcfirst $opt;
-				$opt = 'page' if $opt eq 'PaperSize';
-				next unless exists $d->{$opt};
-				$newdata{$opt} = $val;
+				$superdata{$opt} = $val;
 			}
 			$successfully_set++;
 		}
-		$self-> data( \%newdata);
+		$self-> SUPER::data( \%superdata);
+		$self-> SUPER::data( \%newdata);
 		return $successfully_set;
 	}
 }
 
-package Prima::PS::File;
-use vars qw(@ISA);
-@ISA=q(Prima::PS::Printer);
+package Prima::PS::Printer::NonGUI;
+use base qw(Prima::PS::Printer);
 
 sub profile_default
 {
-	my $def = $_[ 0]-> SUPER::profile_default;
+	my $def  = $_[0]-> SUPER::profile_default;
+	my %def2 = $_[0]-> profile_default2;
+	delete $def2{spoolerType};
 	my %prf = (
-		file => 'out.ps',
 		gui  => 0,
+		%def2,
 	);
 	@$def{keys %prf} = values %prf;
 	return $def;
@@ -556,48 +691,43 @@ sub init
 {
 	my $self = shift;
 	my %profile = $self-> SUPER::init(@_);
-	$self-> {data}-> {spoolerType} = Prima::PS::Printer::file;
-	$self-> {data}-> {spoolerData} = $profile{file};
+	my %def2    = $self-> profile_default2;
+	$self-> {data}-> {spoolerType} = delete $def2{spoolerType};
+	while ( my ( $k, $v ) = each %def2) {
+		$self-> $k( $profile{$k} );
+	}
+	return %profile;
 }
 
+package Prima::PS::File::Common;
+
+sub profile_default2 { ( file => 'out.ps', spoolerType => 'file' ) }
+
 sub file
 {
 	return $_[0]-> {data}-> {spoolerData} unless $#_;
 	$_[0]-> {data}-> {spoolerData} = $_[1];
 }
 
-package Prima::PS::FileHandle;
-use vars qw(@ISA);
-@ISA=q(Prima::PS::Printer);
+package Prima::PS::File;
+use base qw(Prima::PS::Printer::NonGUI Prima::PS::File::Common);
 
-sub profile_default
-{
-	my $def = $_[ 0]-> SUPER::profile_default;
-	my %prf = (
-		handle => undef,
-		gui    => 0,
-	);
-	@$def{keys %prf} = values %prf;
-	return $def;
-}
+package Prima::PS::FileHandle::Common;
 
-sub init
-{
-	my $self = shift;
-	my %profile = $self-> SUPER::init(@_);
-	$self-> {data}-> {spoolerType} = Prima::PS::Printer::fh;
-	$self-> {spoolHandle} = $profile{handle};
-}
+sub profile_default2 { ( handle => undef, spoolerType => 'fh' ) }
 
 sub handle
 {
 	return $_[0]-> {spoolHandle} unless $#_;
 	$_[0]-> {spoolHandle} = $_[1];
+	binmode $_[1] if $_[1];
 }
 
+package Prima::PS::FileHandle;
+use base qw(Prima::PS::Printer::NonGUI Prima::PS::FileHandle::Common);
+
 package Prima::PS::LPR;
-use vars qw(@ISA);
-@ISA=q(Prima::PS::Printer);
+use base q(Prima::PS::Printer);
 
 sub profile_default
 {
@@ -614,8 +744,9 @@ sub init
 {
 	my $self = shift;
 	my %profile = $self-> SUPER::init(@_);
-	$self-> {data}-> {spoolerType} = Prima::PS::Printer::lpr;
+	$self-> {data}-> {spoolerType} = 'lpr';
 	$self-> {data}-> {spoolerData} = $profile{args};
+	return %profile;
 }
 
 sub args
@@ -625,8 +756,7 @@ sub args
 }
 
 package Prima::PS::Pipe;
-use vars qw(@ISA);
-@ISA=q(Prima::PS::Printer);
+use base qw(Prima::PS::Printer);
 
 sub profile_default
 {
@@ -643,8 +773,9 @@ sub init
 {
 	my $self = shift;
 	my %profile = $self-> SUPER::init(@_);
-	$self-> {data}-> {spoolerType} = Prima::PS::Printer::cmd;
+	$self-> {data}-> {spoolerType} = 'cmd';
 	$self-> {data}-> {spoolerData} = $profile{command};
+	return %profile;
 }
 
 sub command
@@ -653,6 +784,98 @@ sub command
 	$_[0]-> {data}-> {spoolerData} = $_[1];
 }
 
+
+package Prima::PS::PDF::Printer;
+use base qw(Prima::PS::PDF Prima::PS::Printer::Common);
+
+Prima::PS::Printer::Common::install_api(__PACKAGE__);
+
+sub profile_default
+{
+	my $def = $_[ 0]-> SUPER::profile_default;
+	my %prf = (
+		gui               => 1,
+		fileExtension     => 'pdf',
+		formatDescription => 'Portable Document Format',
+		defaultData       => {
+			color          => 1,
+			resolution     => 300,
+			page           => 'A4',
+			scaling        => 1,
+			portrait       => 1,
+			spoolerType    => 'file',
+			spoolerData    => '',
+		},
+	);
+	@$def{keys %prf} = values %prf;
+	return $def;
+}
+
+sub init
+{
+	my $self = shift;
+	my %profile = $self-> SUPER::init(@_);
+	$self-> {backend} = 'Prima::PS::PDF';
+	$self-> {defaultData} = {};
+	$self-> {data}        = {%{ $profile{defaultData}}, spoolerType => 'file'};
+	$self-> {$_} = $profile{$_} for qw(gui fileExtension formatDescription);
+	$self-> defaultData( $profile{defaultData} );
+	$self-> data( $profile{data} );
+	return %profile 
+}
+
+sub resolution
+{
+	return $_[0]-> SUPER::resolution unless $#_;
+	$_[0]-> raise_ro("resolution") if @_ != 3; # pass inherited call
+	shift-> SUPER::resolution( @_);
+}
+
+sub printer { $#_ ? 0 : __PACKAGE__ }
+
+sub setup_dialog
+{
+	return unless $_[0]-> {gui};
+	$_[0]-> sdlg_exec(1);
+}
+
+package Prima::PS::PDF::NonGUI;
+use base qw(Prima::PS::PDF::Printer);
+
+sub profile_default
+{
+	my $def  = $_[0]-> SUPER::profile_default;
+	my %def2 = $_[0]-> profile_default2;
+	delete $def2{spoolerType};
+	my %prf = (
+		gui  => 0,
+		%def2,
+	);
+	@$def{keys %prf} = values %prf;
+	return $def;
+}
+
+sub init
+{
+	my $self = shift;
+	my %profile = $self-> SUPER::init(@_);
+	my %def2    = $self-> profile_default2;
+	$self-> {data}-> {spoolerType} = delete $def2{spoolerType};
+	while ( my ( $k, $v ) = each %def2) {
+		$self-> $k( $profile{$k} );
+	}
+
+	return %profile;
+}
+
+package Prima::PS::PDF::File;
+use base qw(Prima::PS::PDF::NonGUI Prima::PS::File::Common);
+
+sub profile_default2 { ( file => 'out.pdf', spoolerType => 'file' ) }
+
+package Prima::PS::PDF::FileHandle;
+use base qw(Prima::PS::PDF::NonGUI Prima::PS::FileHandle::Common);
+
 1;
 
 =pod
@@ -678,6 +901,8 @@ US Common #10 Envelope>.
 
 =item Copies INTEGER
 
+(not applicable to PDF)
+
 =item Scaling FLOAT
 
 1 is 100%, 1.5 is 150%, etc.
@@ -686,25 +911,20 @@ US Common #10 Envelope>.
 
 One of : C<Portrait>, C<Landscape>.
 
-=item UseDeviceFonts BOOLEAN
-
-If 1, use limited set of device fonts in addition to exported bitmap fonts.
-
-=item UseDeviceFontsOnly BOOLEAN
-
-If 1, use limited set of device fonts instead of exported bitmap fonts.
-Its usage may lead to that some document fonts will be mismatched.
-
 =item MediaType STRING
 
 An arbitrary string representing special attributes of the medium other
 than its size, color, and weight. This parameter can be used to identify special
 media such as envelopes, letterheads, or preprinted forms.
 
+(not applicable to PDF)
+
 =item MediaColor STRING
 
 A string identifying the color of the medium.
 
+(not applicable to PDF)
+
 =item MediaWeight FLOAT
 
 The weight of the medium in grams per square meter. "Basis weight" or
@@ -712,6 +932,8 @@ or null "ream weight" in pounds can be converted to grams per square meter by
 multiplying by 3.76; for example, 10-pound paper is approximately 37.6
 grams per square meter.
 
+(not applicable to PDF)
+
 =item MediaClass STRING
 
 (Level 3) An arbitrary string representing attributes of the medium
@@ -723,6 +945,8 @@ The MediaClass entry in the output device dictionary defines the allowable
 values for this parameter on a given device; attempting to set it to an unsupported
 value will cause a configuration error.
 
+(not applicable to PDF)
+
 =item InsertSheet BOOLEAN
 
 (Level 3) A flag specifying whether to insert a sheet of some special
@@ -732,6 +956,8 @@ passing through the device's usual imaging mechanism (such as the fuser
 assembly on a laser printer). Consequently, nothing painted on the current
 page is actually imaged on the inserted medium.
 
+(not applicable to PDF)
+
 =item LeadingEdge BOOLEAN
 
 (Level 3) A value specifying the edge of the input medium that will
@@ -740,6 +966,8 @@ Values reflect positions relative to a canonical page in portrait orientation
 (width smaller than height). When duplex printing is enabled, the canonical
 page orientation refers only to the front (recto) side of the medium.
 
+(not applicable to PDF)
+
 =item ManualFeed BOOLEAN
 
 Flag indicating whether the input medium is to be fed manually by a human
@@ -751,6 +979,8 @@ although their values may be presented to the human operator as an aid in select
 the correct medium. On devices that offer more than one manual feeding mechanism,
 the attributes may select among them.
 
+(not applicable to PDF)
+
 =item TraySwitch BOOLEAN
 
 (Level 3)  A flag specifying whether the output device supports
@@ -759,6 +989,8 @@ runs out of medium, some devices with multiple media sources can switch
 automatically, without human intervention, to an alternate source with the
 same attributes (such as PageSize and MediaColor) as the original.
 
+(not applicable to PDF)
+
 =item MediaPosition STRING
 
 (Level 3) The position number of the media source to be used.
@@ -768,17 +1000,23 @@ satisfy the input media request in a manner consistent with normal media
 selection - even if the media source it specifies is not the best available
 match for the requested attributes.
 
+(not applicable to PDF)
+
 =item DeferredMediaSelection BOOLEAN
 
 (Level 3) A flag determining when to perform media selection.
 If Yes, media will be selected by an independent printing subsystem associated
 with the output device itself.
 
+(not applicable to PDF)
+
 =item MatchAll BOOLEAN
 
 A flag specifying whether input media request should match to all
 non-null values - MediaColor, MediaWeight etc.
 
+(not applicable to PDF)
+
 =back
 
 =head1 AUTHOR
@@ -787,5 +1025,5 @@ Dmitry Karasik, E<lt>dmitry@karasik.eu.orgE<gt>.
 
 =head1 SEE ALSO
 
-L<Prima>, L<Prima::Printer>, L<Prima::Drawable>,
+L<Prima>, L<Prima::Printer>, L<Prima::Drawable>, L<Prima::PS::PDF>
 
