@@ -4,6 +4,7 @@ use strict;
 use warnings;
 use Prima::Classes;
 use Prima::Lists;
+use Prima::Dialog::FindDialog;
 
 package Prima::VB::Divider;
 use strict;
@@ -281,9 +282,11 @@ package Prima::VB::CodeEditor;
 use strict;
 use vars qw(@ISA @editors);
 @ISA = qw(Prima::Window);
+use Prima::sys::FS;
 
 sub profile_default
 {
+	no warnings 'once';
 	my $def = $_[ 0]-> SUPER::profile_default;
 	my %prf = (
 		text           => 'Code editor',
@@ -357,7 +360,7 @@ sub load_code
 	}
 	my $d = VB::open_dialog( filter => [[ 'All files' => '*']]);
 	return unless $d-> execute;
-	unless ( open F, "< " . $d-> fileName) {
+	unless ( open F, "<", $d-> fileName) {
 		Prima::MsgBox::message("Cannot open " . $d-> fileName . ":$!");
 		return;
 	}
@@ -370,7 +373,7 @@ sub save_code
 {
 	my $d = VB::save_dialog( filter => [[ 'All files' => '*']]);
 	return unless $d-> execute;
-	unless ( open F, "> " . $d-> fileName) {
+	unless ( open F, ">", $d-> fileName) {
 		Prima::MsgBox::message("Cannot open " . $d-> fileName . ":$!");
 		return;
 	}
@@ -423,7 +426,7 @@ sub find_dialog
 	my @props = qw(findText options scope);
 	push( @props, q(replaceText)) unless $findStyle;
 	if ( $fd) { for( @props) { $prf{$_} = $fd-> {$_}}}
-	$findDialog = Prima::FindDialog-> create unless $findDialog;
+	$findDialog = Prima::Dialog::FindDialog-> create unless $findDialog;
 	$findDialog-> set( %prf, findStyle => $findStyle, text => ($findStyle ? 'Find' : 'Replace'));
 	$findDialog-> Find-> items($fd-> {findItems});
 	$findDialog-> Replace-> items($fd-> {replaceItems}) unless $findStyle;
