@@ -92,7 +92,7 @@ apc_query_drive_type( const char *drive)
 #ifdef __CYGWIN__
 	return false;
 #endif
-	strncpy( buf, drive, 255);             //     sometimes D: isn't enough for 95,
+	strlcpy( buf, drive, 255);             //     sometimes D: isn't enough for 95,
 	if ( buf[1] == ':' && buf[2] == 0) {   //     but ok for D:\.
 		buf[2] = '\\';                      //
 		buf[3] = 0;                         //
@@ -218,8 +218,8 @@ apc_sys_get_value( int sysValue)
 	case svLayeredWidgets: return guts. displayBMInfo. bmiHeader. biBitCount > 8;
 	case svFixedPointerSize: return 0;
 	case svMenuCheckSize   : return GetSystemMetrics( SM_CXMENUCHECK );
-	case svFriBidi         : return use_fribidi;
-	case svLibThai         : return use_libthai;
+	case svFriBidi         : return prima_guts.use_fribidi;
+	case svLibThai         : return prima_guts.use_libthai;
 	case svAntialias       : return 1;
 	default:
 		return -1;
@@ -352,7 +352,7 @@ apc_fetch_resource( const char *className, const char *name,
 			duplicate_string( PComponent( owner)-> name), false), 0);
 		list_insert_at(&ids[0], ( Handle) prima_normalize_resource_string(
 			duplicate_string(
-				( owner == application) ? "Prima" : CComponent( owner)-> className
+				( owner == prima_guts.application) ? "Prima" : CComponent( owner)-> className
 			), true), 0);
 		owner = PComponent( owner)-> owner;
 	}
